@@ -6,13 +6,13 @@ The Python implementation of the Zero Install injector is divided into four sub-
  - L{zeroinstall.gtkui} contains code for making GTK user-interfaces
  - L{zeroinstall.support} contains helper code (not really specific to Zero Install)
 
-@copyright: (C) 2009, Thomas Leonard
+@copyright: (C) 2010, Thomas Leonard
 @see: U{http://0install.net}
 
 @var _: a function for translating strings using the zero-install domain (for use internally by Zero Install)
 """
 
-version = '0.49'
+version = '0.51'
 
 import gettext
 from os.path import dirname, join
@@ -21,7 +21,7 @@ try:
 	localedir = None
 	translation = gettext.translation('zero-install', fallback = False)
 except:
-	localedir = join(dirname(dirname(__file__)), 'locale')
+	localedir = join(dirname(dirname(__file__)), 'share', 'locale')
 	translation = gettext.translation('zero-install',
 				localedir = localedir,
 				fallback = True)
