@@ -7,10 +7,10 @@ Integration with native distribution package managers.
 # See the README file for details, or visit http://0install.net.
 
 from zeroinstall import _
-import os, re, glob, subprocess, sys
+import os, platform, re, glob, subprocess, sys
 from logging import warn, info
 from zeroinstall.injector import namespaces, model, arch
-from zeroinstall.support import basedir
+from zeroinstall.support import basedir, tasks
 
 _dotted_ints = '[0-9]+(?:\.[0-9]+)*'
 
@@ -120,6 +120,9 @@ def try_cleanup_distro_version(version):
 	We do this by stripping off anything we can't parse.
 	@return: the part we understood, or None if we couldn't parse anything
 	@rtype: str"""
+	if ':' in version:
+		version = version.split(':')[1]	# Skip 'epoch'
+	version = version.replace('_', '-')
 	match = re.match(_version_regexp, version)
 	if match:
 		version, revision = match.groups()
@@ -135,6 +138,7 @@ class Distribution(object):
 	particular distributions. This base class ignores the native package manager.
 	@since: 0.28
 	"""
+	_packagekit = None
 
 	def get_package_info(self, package, factory):
 		"""Get information about the given package.
@@ -159,12 +163,65 @@ class Distribution(object):
 		"""
 		return 0
 
-	def get_installed(self, package_id):
-		"""Check whether 'package' is currently installed.
-		@param package_id: the Implementation ID used by get_package_info
-		@type package_id: str
-		@return: True iff the package is currently installed"""
-		return True
+	def get_feed(self, master_feed):
+		"""Generate a feed containing information about distribution packages.
+		This should immediately return a feed containing an implementation for the
+		package if it's already installed. Information about versions that could be
+		installed using the distribution's package manager can be added asynchronously
+		later (see L{fetch_candidates}).
+		@param master_feed: feed containing the <package-implementation> elements
+		@type master_feed: L{model.ZeroInstallFeed}
+		@rtype: L{model.ZeroInstallFeed}"""
+
+		feed = model.ZeroInstallFeed(None)
+		feed.url = 'distribution:' + master_feed.url
+
+		for item, item_attrs in master_feed.get_package_impls(self):
+			package = item_attrs.get('package', None)
+			if package is None:
+				raise model.InvalidInterface(_("Missing 'package' attribute on %s") % item)
+
+			def factory(id, only_if_missing = False, installed = True):
+				assert id.startswith('package:')
+				if id in feed.implementations:
+					if only_if_missing:
+						return None
+					warn(_("Duplicate ID '%s' for DistributionImplementation"), id)
+				impl = model.DistributionImplementation(feed, id, self)
+				feed.implementations[id] = impl
+
+				impl.installed = installed
+				impl.metadata = item_attrs
+
+				item_main = item_attrs.get('main', None)
+				if item_main and not item_main.startswith('/'):
+					raise model.InvalidInterface(_("'main' attribute must be absolute, but '%s' doesn't start with '/'!") %
+								item_main)
+				impl.main = item_main
+				impl.upstream_stability = model.packaged
+
+				return impl
+
+			self.get_package_info(package, factory)
+		return feed
+
+	def fetch_candidates(self, master_feed):
+		"""Collect information about versions we could install using
+		the distribution's package manager. On success, the distribution
+		feed in iface_cache is updated.
+		@return: a L{tasks.Blocker} if the task is in progress, or None if not"""
+		if self.packagekit.available:
+			package_names = [item.getAttribute("package") for item, item_attrs in master_feed.get_package_impls(self)]
+			return self.packagekit.fetch_candidates(package_names)
+
+	@property
+	def packagekit(self):
+		"""For use by subclasses.
+		@rtype: L{packagekit.PackageKit}"""
+		if not self._packagekit:
+			from zeroinstall.injector import packagekit
+			self._packagekit = packagekit.PackageKit()
+		return self._packagekit
 
 class CachedDistribution(Distribution):
 	"""For distributions where querying the package database is slow (e.g. requires running
@@ -259,7 +316,7 @@ _canonical_machine = {
 	'ppc': 'ppc',
 }
 
-host_machine = arch.canonicalize_machine(os.uname()[-1])
+host_machine = arch.canonicalize_machine(platform.uname()[4])
 def canonical_machine(package_machine):
 	machine = _canonical_machine.get(package_machine, None)
 	if machine is None:
@@ -274,7 +331,7 @@ class DebianDistribution(Distribution):
 
 	def __init__(self, dpkg_status, pkgcache):
 		self.dpkg_cache = Cache('dpkg-status.cache', dpkg_status, 2)
-		self.apt_cache = Cache('apt-cache-cache', pkgcache, 3)
+		self.apt_cache = {}
 
 	def _query_installed_package(self, package):
 		null = os.open('/dev/null', os.O_WRONLY)
@@ -287,9 +344,6 @@ class DebianDistribution(Distribution):
 			if not line: continue
 			version, debarch, status = line.split('\t', 2)
 			if not status.endswith(' installed'): continue
-			if ':' in version:
-				# Debian's 'epoch' system
-				version = version.split(':', 1)[1]
 			clean_version = try_cleanup_distro_version(version)
 			if clean_version:
 				return '%s\t%s' % (clean_version, canonical_machine(debarch.strip()))
@@ -299,58 +353,37 @@ class DebianDistribution(Distribution):
 		return '-'
 
 	def get_package_info(self, package, factory):
+		# Add any already-installed package...
 		installed_cached_info = self._get_dpkg_info(package)
 
 		if installed_cached_info != '-':
 			installed_version, machine = installed_cached_info.split('\t')
-			impl = factory('package:deb:%s:%s' % (package, installed_version))
+			impl = factory('package:deb:%s:%s:%s' % (package, installed_version, machine))
 			impl.version = model.parse_version(installed_version)
 			if machine != '*':
 				impl.machine = machine
 		else:
 			installed_version = None
 
-		# Check to see whether we could get a newer version using apt-get
+		# Add any uninstalled candidates (note: only one of these two methods will add anything)
 
-		cached = self.apt_cache.get(package)
-		if cached is None:
-			try:
-				null = os.open('/dev/null', os.O_WRONLY)
-				child = subprocess.Popen(['apt-cache', 'show', '--no-all-versions', '--', package], stdout = subprocess.PIPE, stderr = null)
-				os.close(null)
+		# From PackageKit...
+		self.packagekit.get_candidates(package, factory, 'package:deb')
 
-				arch = version = size = None
-				for line in child.stdout:
-					line = line.strip()
-					if line.startswith('Version: '):
-						version = line[9:]
-						if ':' in version:
-							# Debian's 'epoch' system
-							version = version.split(':', 1)[1]
-						version = try_cleanup_distro_version(version)
-					elif line.startswith('Architecture: '):
-						arch = canonical_machine(line[14:].strip())
-					elif line.startswith('Size: '):
-						size = int(line[6:].strip())
-				if version and arch:
-					cached = '%s\t%s\t%d' % (version, arch, size)
-				else:
-					cached = '-'
-				child.wait()
-			except Exception, ex:
-				warn("'apt-cache show %s' failed: %s", package, ex)
-				cached = '-'
-			# (multi-arch support? can there be multiple candidates?)
-			self.apt_cache.put(package, cached)
-
-		if cached != '-':
-			candidate_version, candidate_arch, candidate_size = cached.split('\t')
+		# From apt-cache...
+		cached = self.apt_cache.get(package, None)
+		if cached:
+			candidate_version = cached['version']
+			candidate_arch = cached['arch']
 			if candidate_version and candidate_version != installed_version:
-				impl = factory('package:deb:%s:%s' % (package, candidate_version))
+				impl = factory('package:deb:%s:%s:%s' % (package, candidate_version, candidate_arch), installed = False)
 				impl.version = model.parse_version(candidate_version)
 				if candidate_arch != '*':
 					impl.machine = candidate_arch
-				impl.download_sources.append(model.DistributionSource(package, candidate_size))
+				def install(handler):
+					raise model.SafeException(_("This program depends on '%s', which is a package that is available through your distribution. "
+							"Please install it manually using your distribution's tools and try again.") % package)
+				impl.download_sources.append(model.DistributionSource(package, cached['size'], install, needs_confirmation = False))
 
 	def get_score(self, disto_name):
 		return int(disto_name == 'Debian')
@@ -363,15 +396,40 @@ class DebianDistribution(Distribution):
 
 		return installed_cached_info
 
-	def get_installed(self, package_id):
-		details = package_id.split(':', 3)
-		assert details[0] == 'package'
-		package = details[2]
-		info = self._get_dpkg_info(package)
-		if info is '-': return False
-		installed_version, machine = info.split('\t')
-		installed_id = 'package:deb:%s:%s' % (package, installed_version)
-		return package_id == installed_id
+	def fetch_candidates(self, master_feed):
+		package_names = [item.getAttribute("package") for item, item_attrs in master_feed.get_package_impls(self)]
+
+		if self.packagekit.available:
+			return self.packagekit.fetch_candidates(package_names)
+
+		# No PackageKit. Use apt-cache directly.
+		for package in package_names:
+			# Check to see whether we could get a newer version using apt-get
+			try:
+				null = os.open('/dev/null', os.O_WRONLY)
+				child = subprocess.Popen(['apt-cache', 'show', '--no-all-versions', '--', package], stdout = subprocess.PIPE, stderr = null)
+				os.close(null)
+
+				arch = version = size = None
+				for line in child.stdout:
+					line = line.strip()
+					if line.startswith('Version: '):
+						version = line[9:]
+						version = try_cleanup_distro_version(version)
+					elif line.startswith('Architecture: '):
+						arch = canonical_machine(line[14:].strip())
+					elif line.startswith('Size: '):
+						size = int(line[6:].strip())
+				if version and arch:
+					cached = {'version': version, 'arch': arch, 'size': size}
+				else:
+					cached = None
+				child.wait()
+			except Exception, ex:
+				warn("'apt-cache show %s' failed: %s", package, ex)
+				cached = None
+			# (multi-arch support? can there be multiple candidates?)
+			self.apt_cache[package] = cached
 
 class RPMDistribution(CachedDistribution):
 	"""An RPM-based distribution."""
@@ -395,10 +453,8 @@ class RPMDistribution(CachedDistribution):
 		self._write_cache(cache)
 
 	def get_package_info(self, package, factory):
-		try:
-			versions = self.versions[package]
-		except KeyError:
-			return
+		# Add installed versions...
+		versions = self.versions.get(package, [])
 
 		for version, machine in versions:
 			impl = factory('package:rpm:%s:%s:%s' % (package, version, machine))
@@ -406,6 +462,9 @@ class RPMDistribution(CachedDistribution):
 			if machine != '*':
 				impl.machine = machine
 
+		# Add any uninstalled candidates found by PackageKit
+		self.packagekit.get_candidates(package, factory, 'package:rpm')
+
 	def get_score(self, disto_name):
 		return int(disto_name == 'RPM')
 
@@ -416,6 +475,7 @@ class SlackDistribution(Distribution):
 		self._packages_dir = packages_dir
 
 	def get_package_info(self, package, factory):
+		# Add installed versions...
 		for entry in os.listdir(self._packages_dir):
 			name, version, arch, build = entry.rsplit('-', 3)
 			if name == package:
@@ -431,6 +491,9 @@ class SlackDistribution(Distribution):
 				if zi_arch != '*':
 					impl.machine = zi_arch
 
+		# Add any uninstalled candidates found by PackageKit
+		self.packagekit.get_candidates(package, factory, 'package:slack')
+
 	def get_score(self, disto_name):
 		return int(disto_name == 'Slack')
 
@@ -440,6 +503,7 @@ class GentooDistribution(Distribution):
 		self._pkgdir = pkgdir
 
 	def get_package_info(self, package, factory):
+		# Add installed versions...
 		_version_start_reqexp = '-[0-9]'
 
 		if package.count('/') != 1: return
@@ -472,6 +536,9 @@ class GentooDistribution(Distribution):
 				impl.version = model.parse_version(version)
 				impl.machine = machine
 
+		# Add any uninstalled candidates found by PackageKit
+		self.packagekit.get_candidates(package, factory, 'package:gentoo')
+
 	def get_score(self, disto_name):
 		return int(disto_name == 'Gentoo')
 
@@ -481,21 +548,24 @@ class PortsDistribution(Distribution):
 		self._pkgdir = pkgdir
 
 	def get_package_info(self, package, factory):
-		_version_start_reqexp = '-[0-9]'
+		_name_version_regexp = '^(.+)-([^-]+)$'
 
+		nameversion = re.compile(_name_version_regexp)
 		for pkgname in os.listdir(self._pkgdir):
 			pkgdir = os.path.join(self._pkgdir, pkgname)
 			if not os.path.isdir(pkgdir): continue
 
 			#contents = file(os.path.join(pkgdir, '+CONTENTS')).readline().strip()
 
-			match = re.search(_version_start_reqexp, pkgname)
+			match = nameversion.search(pkgname)
 			if match is None:
-				warn(_('Cannot parse version from Ports package named "%(pkgname)s"'), {'name': pkgname})
+				warn(_('Cannot parse version from Ports package named "%(pkgname)s"'), {'pkgname': pkgname})
 				continue
 			else:
-				name = pkgname[0:match.start()]
-				version = try_cleanup_distro_version(pkgname[match.start() + 1:])
+				name = match.group(1)
+				if name != package:
+					continue
+				version = try_cleanup_distro_version(match.group(2))
 
 			machine = host_machine
 
