@@ -18,6 +18,7 @@ import os, re, locale
 from logging import info, debug, warn
 from zeroinstall import SafeException, version
 from zeroinstall.injector.namespaces import XMLNS_IFACE
+from zeroinstall.injector import qdom
 
 # Element names for bindings in feed files
 binding_names = frozenset(['environment', 'overlay'])
@@ -37,6 +38,8 @@ defaults = {
 
 class InvalidInterface(SafeException):
 	"""Raised when parsing an invalid feed."""
+	feed_url = None
+
 	def __init__(self, message, ex = None):
 		if ex:
 			try:
@@ -52,6 +55,11 @@ class InvalidInterface(SafeException):
 
 		SafeException.__init__(self, message)
 
+	def __str__(self):
+		if self.feed_url:
+			return SafeException.__str__(self) + ' in ' + self.feed_url
+		return SafeException.__str__(self)
+
 def _split_arch(arch):
 	"""Split an arch into an (os, machine) tuple. Either or both parts may be None."""
 	if not arch:
@@ -116,13 +124,21 @@ def process_binding(e):
 	else:
 		raise Exception(_("Unknown binding type '%s'") % e.name)
 
-def process_depends(item):
+def process_depends(item, local_feed_dir):
 	"""Internal"""
 	# Note: also called from selections
+	attrs = item.attrs
 	dep_iface = item.getAttribute('interface')
 	if not dep_iface:
-		raise InvalidInterface(_("Missing 'interface' on <requires>"))
-	dependency = InterfaceDependency(dep_iface, metadata = item.attrs)
+		raise InvalidInterface(_("Missing 'interface' on <%s>") % item.name)
+	if dep_iface.startswith('./'):
+		if local_feed_dir:
+			dep_iface = os.path.abspath(os.path.join(local_feed_dir, dep_iface))
+			# (updates the element too, in case we write it out again)
+			attrs['interface'] = dep_iface
+		else:
+			raise InvalidInterface(_('Relative interface URI "%s" in non-local feed') % dep_iface)
+	dependency = InterfaceDependency(dep_iface, element = item)
 
 	for e in item.childNodes:
 		if e.uri != XMLNS_IFACE: continue
@@ -306,17 +322,20 @@ class Feed(object):
 class Dependency(object):
 	"""A Dependency indicates that an Implementation requires some additional
 	code to function. This is an abstract base class.
+	@ivar qdom: the XML element for this Dependency (since 0launch 0.51)
+	@type qdom: L{qdom.Element}
 	@ivar metadata: any extra attributes from the XML element
 	@type metadata: {str: str}
 	"""
-	__slots__ = ['metadata']
+	__slots__ = ['qdom']
 
-	def __init__(self, metadata):
-		if metadata is None:
-			metadata = {}
-		else:
-			assert not isinstance(metadata, basestring)	# Use InterfaceDependency instead!
-		self.metadata = metadata
+	def __init__(self, element):
+		assert isinstance(element, qdom.Element), type(element)	# Use InterfaceDependency instead!
+		self.qdom = element
+
+	@property
+	def metadata(self):
+		return self.qdom.attrs
 
 class InterfaceDependency(Dependency):
 	"""A Dependency on a Zero Install interface.
@@ -328,10 +347,10 @@ class InterfaceDependency(Dependency):
 	@type bindings: [L{Binding}]
 	@since: 0.28
 	"""
-	__slots__ = ['interface', 'restrictions', 'bindings', 'metadata']
+	__slots__ = ['interface', 'restrictions', 'bindings']
 
-	def __init__(self, interface, restrictions = None, metadata = None):
-		Dependency.__init__(self, metadata)
+	def __init__(self, interface, restrictions = None, element = None):
+		Dependency.__init__(self, element)
 		assert isinstance(interface, (str, unicode))
 		assert interface
 		self.interface = interface
@@ -340,7 +359,7 @@ class InterfaceDependency(Dependency):
 		else:
 			self.restrictions = restrictions
 		self.bindings = []
-	
+
 	def __str__(self):
 		return _("<Dependency on %(interface)s; bindings: %(bindings)s%(restrictions)s>") % {'interface': self.interface, 'bindings': self.bindings, 'restrictions': self.restrictions}
 
@@ -393,6 +412,47 @@ class DistributionSource(RetrievalMethod):
 		self.install = install
 		self.needs_confirmation = needs_confirmation
 
+class Command(object):
+	"""A Command is a way of running an Implementation as a program."""
+
+	__slots__ = ['qdom', '_depends', '_local_dir', '_runner']
+
+	def __init__(self, qdom, local_dir):
+		"""@param qdom: the <command> element
+		@param local_dir: the directory containing the feed (for relative dependencies), or None if not local
+		"""
+		assert qdom.name == 'command', 'not <command>: %s' % qdom
+		self.qdom = qdom
+		self._local_dir = local_dir
+		self._depends = None
+
+	path = property(lambda self: self.qdom.attrs.get("path", None))
+
+	def _toxml(self, doc, prefixes):
+		return self.qdom.toDOM(doc, prefixes)
+
+	@property
+	def requires(self):
+		if self._depends is None:
+			self._runner = None
+			depends = []
+			for child in self.qdom.childNodes:
+				if child.name == 'requires':
+					dep = process_depends(child, self._local_dir)
+					depends.append(dep)
+				elif child.name == 'runner':
+					if self._runner:
+						raise InvalidInterface(_("Multiple <runner>s in <command>!"))
+					dep = process_depends(child, self._local_dir)
+					depends.append(dep)
+					self._runner = dep
+			self._depends = depends
+		return self._depends
+
+	def get_runner(self):
+		self.requires		# (sets _runner)
+		return self._runner
+
 class Implementation(object):
 	"""An Implementation is a package which implements an Interface.
 	@ivar download_sources: list of methods of getting this implementation
@@ -409,7 +469,8 @@ class Implementation(object):
 	@type langs: str
 	@ivar requires: interfaces this package depends on
 	@type requires: [L{Dependency}]
-	@ivar main: the default file to execute when running as a program
+	@ivar commands: ways to execute as a program
+	@type commands: {str: Command}
 	@ivar metadata: extra metadata from the feed
 	@type metadata: {"[URI ]localName": str}
 	@ivar id: a unique identifier for this Implementation
@@ -424,14 +485,13 @@ class Implementation(object):
 	# Note: user_stability shouldn't really be here
 
 	__slots__ = ['upstream_stability', 'user_stability', 'langs',
-		     'requires', 'main', 'metadata', 'download_sources',
+		     'requires', 'metadata', 'download_sources', 'commands',
 		     'id', 'feed', 'version', 'released', 'bindings', 'machine']
 
 	def __init__(self, feed, id):
 		assert id
 		self.feed = feed
 		self.id = id
-		self.main = None
 		self.user_stability = None
 		self.upstream_stability = None
 		self.metadata = {}	# [URI + " "] + localName -> value
@@ -442,6 +502,7 @@ class Implementation(object):
 		self.langs = ""
 		self.machine = None
 		self.bindings = []
+		self.commands = {}
 
 	def get_stability(self):
 		return self.user_stability or self.upstream_stability or testing
@@ -475,6 +536,21 @@ class Implementation(object):
 	digests = None
 	requires_root_install = False
 
+	def _get_main(self):
+		""""@deprecated: use commands["run"] instead"""
+		main = self.commands.get("run", None)
+		if main is not None:
+			return main.path
+		return None
+	def _set_main(self, path):
+		""""@deprecated: use commands["run"] instead"""
+		if path is None:
+			if "run" in self.commands:
+				del self.commands["run"]
+		else:
+			self.commands["run"] = Command(qdom.Element(XMLNS_IFACE, 'command', {'path': path}), None)
+	main = property(_get_main, _set_main)
+
 class DistributionImplementation(Implementation):
 	"""An implementation provided by the distribution. Information such as the version
 	comes from the package manager.
@@ -717,7 +793,7 @@ class ZeroInstallFeed(object):
 		if not self.summary:
 			raise InvalidInterface(_("Missing <summary> in feed"))
 
-		def process_group(group, group_attrs, base_depends, base_bindings):
+		def process_group(group, group_attrs, base_depends, base_bindings, base_commands):
 			for item in group.childNodes:
 				if item.uri != XMLNS_IFACE: continue
 
@@ -726,6 +802,7 @@ class ZeroInstallFeed(object):
 
 				depends = base_depends[:]
 				bindings = base_bindings[:]
+				commands = base_commands.copy()
 
 				item_attrs = _merge_attrs(group_attrs, item)
 
@@ -738,15 +815,30 @@ class ZeroInstallFeed(object):
 				for child in item.childNodes:
 					if child.uri != XMLNS_IFACE: continue
 					if child.name == 'requires':
-						dep = process_depends(child)
+						dep = process_depends(child, local_dir)
 						depends.append(dep)
+					elif child.name == 'command':
+						command_name = child.attrs.get('name', None)
+						if not command_name:
+							raise InvalidInterface('Missing name for <command>')
+						commands[command_name] = Command(child, local_dir)
 					elif child.name in binding_names:
 						bindings.append(process_binding(child))
 
+				for attr, command in [('main', 'run'),
+						      ('self-test', 'test')]:
+					value = item.attrs.get(attr, None)
+					if value is not None:
+						commands[command] = Command(qdom.Element(XMLNS_IFACE, 'command', {'path': value}), None)
+
+				compile_command = item.attrs.get('http://zero-install.sourceforge.net/2006/namespaces/0compile command')
+				if compile_command is not None:
+					commands['compile'] = Command(qdom.Element(XMLNS_IFACE, 'command', {'shell-command': compile_command}), None)
+
 				if item.name == 'group':
-					process_group(item, item_attrs, depends, bindings)
+					process_group(item, item_attrs, depends, bindings, commands)
 				elif item.name == 'implementation':
-					process_impl(item, item_attrs, depends, bindings)
+					process_impl(item, item_attrs, depends, bindings, commands)
 				elif item.name == 'package-implementation':
 					if depends:
 						warn("A <package-implementation> with dependencies in %s!", self.url)
@@ -754,7 +846,7 @@ class ZeroInstallFeed(object):
 				else:
 					assert 0
 
-		def process_impl(item, item_attrs, depends, bindings):
+		def process_impl(item, item_attrs, depends, bindings, commands):
 			id = item.getAttribute('id')
 			if id is None:
 				raise InvalidInterface(_("Missing 'id' attribute on %s") % item)
@@ -786,11 +878,8 @@ class ZeroInstallFeed(object):
 				raise InvalidInterface(_("Missing version attribute"))
 			impl.version = parse_version(version)
 
-			item_main = item_attrs.get('main', None)
-			if item_main and item_main.startswith('/'):
-				raise InvalidInterface(_("'main' attribute must be relative, but '%s' starts with '/'!") %
-							item_main)
-			impl.main = item_main
+			item_main = commands.get('run', None)
+			impl.commands = commands
 
 			impl.released = item_attrs.get('released', None)
 			impl.langs = item_attrs.get('langs', '')
@@ -851,9 +940,11 @@ class ZeroInstallFeed(object):
 						impl.download_sources.append(recipe)
 
 		root_attrs = {'stability': 'testing'}
+		root_commands = {}
 		if main:
-			root_attrs['main'] = main
-		process_group(feed_element, root_attrs, [], [])
+			info("Note: @main on document element is deprecated in %s", self)
+			root_commands['run'] = Command(qdom.Element(XMLNS_IFACE, 'command', {'path': main}), None)
+		process_group(feed_element, root_attrs, [], [], root_commands)
 	
 	def get_distro_feed(self):
 		"""Does this feed contain any <pacakge-implementation> elements?
@@ -973,6 +1064,7 @@ def _pretty_escape(uri):
 def canonical_iface_uri(uri):
 	"""If uri is a relative path, convert to an absolute one.
 	A "file:///foo" URI is converted to "/foo".
+	An "alias:prog" URI expands to the URI in the 0alias script
 	Otherwise, return it unmodified.
 	@rtype: str
 	@raise SafeException: if uri isn't valid
@@ -983,6 +1075,17 @@ def canonical_iface_uri(uri):
 		return uri
 	elif uri.startswith('file:///'):
 		return uri[7:]
+	elif uri.startswith('alias:'):
+		from zeroinstall import alias, support
+		alias_prog = uri[6:]
+		if not os.path.isabs(alias_prog):
+			full_path = support.find_in_path(alias_prog)
+			if not full_path:
+				raise alias.NotAnAliasScript("Not found in $PATH: " + alias_prog)
+		else:
+			full_path = alias_prog
+		interface_uri, main = alias.parse_script(full_path)
+		return interface_uri
 	else:
 		iface_uri = os.path.realpath(uri)
 		if os.path.isfile(iface_uri):
